# Copyright 2019 Arie Bregman
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import logging
import sys

import infraform.cli.parser as app_parser
from infraform.exceptions.usage import general_usage

LOG = logging.getLogger(__name__)


def setup_logging(debug):
    """Sets the logging."""
    format = '%(message)s'
    level = logging.DEBUG if debug else logging.INFO
    logging.basicConfig(level=level, format=format)


def main():
    """Main Entry."""
    # Parse arguments provided by the user
    parser = app_parser.create_parser()
    args = parser.parse_args()
    setup_logging(args.debug)

    if hasattr(args, 'func'):
        args.func(args)
    else:
        LOG.error(general_usage())


if __name__ == '__main__':
    sys.exit(main())



# Copyright 2019 Arie Bregman
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import os
import re

from difflib import SequenceMatcher
import logging
import subprocess
import sys
from ansible.parsing.splitter import split_args, parse_kv
import crayons
import jinja2 as j2
import yaml

from infraform import filters
from infraform.exceptions import requirements as req_exc
from infraform.exceptions import usage as usage_exc
from infraform.exceptions.utils import success_or_exit

LOG = logging.getLogger(__name__)


class Platform(object):

    SCENARIOS_PATH = os.path.dirname(__file__) + '/../scenarios'

    def __init__(self, args):
        self.args = {k: v for k, v in vars(args).items() if v is not None}
        # vars are used for feeding scenario templates (jinja2)
        if 'vars' in self.args:
            self.vars = self.get_vars(self.args['vars'])
        self.check_platform_avaiable()

        # If user specified scenario, make sure it exists
        if 'scenario' in self.args:
            self.scenario_fpath, self.scenario_f = self.verify_scenario_exists(
                self.SCENARIOS_PATH, self.args['scenario'])
            self.scenario_dir = os.path.dirname(
                self.scenario_fpath).split('/')[-1]

            self.render_scenario()

            _, suffix = os.path.splitext(self.scenario_f)
            if suffix == ".yml" or suffix == ".yaml":
                self.load_yaml_to_vars()

        self.create_new_vars()
        self.create_workspace_dir()

    def create_workspace_dir(self):
        """Create infraform workspace."""
        if not os.path.exists('.infraform'):
            os.mkdir('.infraform')

    def load_yaml_to_vars(self):
        """Load any scenario YAML directives to variables."""
        with open(self.scenario_f, 'r') as stream:
            try:
                scenario_yaml = yaml.safe_load(stream)
                self.vars['scenario_vars'] = {
                    k: v for k, v in scenario_yaml.items() if v is not None}
                for k, v in scenario_yaml.items():
                    if k not in self.vars:
                        self.vars.update({k: v})
            except yaml.YAMLError as exc:
                print(exc)

    def create_new_vars(self):
        """Create additional variables out of existing variables."""
        if 'project' in self.vars:
            if '/' in self.vars['project']:
                self.vars['project_name'] = os.path.basename(
                    self.vars['project'])
            else:
                self.vars['project_name'] = self.vars['project']

    def get_vars(self, args):
        """Updates variables based on given arguments from CLI."""
        variables = {}
        args_split = split_args(args)
        for arg in args_split:
            variables.update(parse_kv(arg))
        return variables

    def check_platform_avaiable(self):
        """Validates the platform specified is ready for use."""
        res = subprocess.run("{} --version".format(self.binary), shell=True,
                             stderr=subprocess.PIPE, stdout=subprocess.PIPE)
        success_or_exit(res.returncode,
                        req_exc.service_down(self.installation))

    @staticmethod
    def verify_scenario_exists(scenarios_dir, scenario):
        """Verifies scenario exists."""
        similar = []
        for p, d, files in os.walk(scenarios_dir):
            for f in files:
                until_dot_pattern = re.compile(r"^[^.]*")
                file_without_suffix = re.search(until_dot_pattern, f).group(0)
                file_name = f
                if f.endswith('.j2'):
                    file_name = f[:-3]
                if file_without_suffix == scenario:
                    scenario_file_path = p + '/' + f
                    scenario_file = file_name
                    return scenario_file_path, scenario_file
                elif SequenceMatcher(None, file_without_suffix,
                                     scenario).ratio() >= 0.25:
                    similar.append(file_without_suffix)
        if similar:
            LOG.info("Maybe you meant:\n\n{}".format(
                crayons.yellow("\n".join(similar))))
        success_or_exit(1, usage_exc.missing_scenario(scenario))

    @staticmethod
    def execute_cmd(cmd, cwd=None):
        """Executes a given command in the specified path."""
        subprocess.run(cmd, shell=True, cwd=cwd)

    def get_template(self, name):
        """Returns jinja2 template."""
        with open(name, 'r+') as open_f:
            template_content = open_f.read()
        return template_content

    def write_rendered_scenario(self, scenario):
        """Save the rendered scenario."""
        print(os.getcwd())
        with open('./' + self.scenario_f, 'w+') as f:
            f.write(scenario)

    def render_scenario(self):
        """Render a scenario and save to disk."""
        j2_env = j2.Environment(loader=j2.FunctionLoader(
            self.get_template), trim_blocks=True, undefined=j2.StrictUndefined)
        j2_env.filters['env_override'] = filters.env_override
        template = j2_env.get_template(self.scenario_fpath)
        try:
            rendered_scenario = template.render(vars=self.vars)
        except j2.exceptions.UndefinedError as e:
            LOG.error(e)
            missing_arg = re.findall(
                r'no attribute (.*)', e.message)[0].strip("'")
            LOG.error(usage_exc.missing_arg(missing_arg))
            sys.exit(2)
        self.write_rendered_scenario(rendered_scenario)



# Copyright 2019 Arie Bregman
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import logging

from infraform.platforms.platform import Platform

LOG = logging.getLogger(__name__)


class Shell(Platform):

    PACKAGE = 'bash'
    BINARY = 'bash'

    def __init__(self, args):
        self.binary = self.BINARY
        self.package = self.PACKAGE
        self.installation = "yum install bash"
        super(Shell, self).__init__(args)

    def prepare(self):
        self.render_scenario()

    def run(self):
        if 'host' in self.vars:
            self.execute_script_remotely()
        else:
            self.execute_cmd("chmod +x {0}; ./{0}".format(self.scenario_f))



# Copyright 2019 Arie Bregman
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import argparse

from infraform.cli.show import parser as show_parser
from infraform.cli.run import parser as run_parser
from infraform.cli.list import parser as list_parser


def create_parser():
    """Returns argument parser"""

    # Top level parser
    parser = argparse.ArgumentParser()
    subparsers = parser.add_subparsers()

    parser.add_argument('--debug', '-d', action='store_true',
                        dest="debug", help='Turn on debug')

    run_parser.add_run_parser(subparsers)
    list_parser.add_list_parser(subparsers)
    show_parser.add_show_parser(subparsers)

    return parser



# Copyright 2019 Arie Bregman
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import re


def get_match_until_first_dot(string):
    """Returns the matched pattern until first dot.

    For example: 'scenario1.py.j2' -> 'scenario'
    """
    until_dot_pattern = re.compile(r"^[^.]*")
    return re.search(until_dot_pattern, string).group(0)


def get_description(f):
    desc = re.findall("description:(.*)", f.read())
    if desc:
        return desc[0]
    else:
        return "-"



# Copyright 2019 Arie Bregman
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from infraform.cli.list import cli as list_cli


def add_list_parser(subparsers):
    """The parser for sub command 'list'."""
    list_parser = subparsers.add_parser("list")
    list_parser.set_defaults(func=list_cli.main)
    list_parser.add_argument('--scenarios', '-s',
                             dest="scenarios",
                             action='store_true',
                             help='List Scenarios')



# Copyright 2019 Arie Bregman
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from infraform.cli.run import cli as run_cli


def add_run_parser(subparsers):
    """The parser for sub command 'run'."""
    run_parser = subparsers.add_parser("run")
    run_parser.set_defaults(func=run_cli.main)
    run_parser.add_argument('--scenario', '-s',
                            dest="scenario",
                            help='Predefined scenario to use for exection')
    run_parser.add_argument('--platform', dest="platform",
                            help="The platform to use \
(podman, docker, terraform, shell, python)")
    run_parser.add_argument('--vars', dest="vars",
                            default="",
                            help="extra variables")



# Copyright 2019 Arie Bregman
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import crayons
import logging
import os

from infraform.utils import get_match_until_first_dot

LOG = logging.getLogger(__name__)

SCENARIOS_PATH = os.path.dirname(__file__) + '/../infraform/scenarios'


def show_scenario(scenario):
    found = False
    for (dirpath, dirnames, filenames) in os.walk(SCENARIOS_PATH):
        for f in filenames:
            name = get_match_until_first_dot(f)
            if name == scenario:
                scenario_path = dirpath + '/' + f
                with open(scenario_path, 'r') as f:
                    LOG.info(f.read())
                    found = True
    if not found:
        LOG.warning("Scenario not found: {}".format(crayons.red(scenario)))



# Copyright 2019 Arie Bregman
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import crayons


def general_usage():
    """Returns general usage string."""
    message = """
Choose one of the following infraform commands: {}
Usage Examples:

    Provision an OpenStack instance with floating IP:
    $ {}

    Run Python PEP8 tests inside a Podman container:
    $ {}

    List available scenarios:
    $ {}

    Show scenario:
    $ {}

    Run unit tests for nova project from git:
    $ {}

""".format(crayons.red("run, list, show"),
           crayons.yellow("infraform run --scenario \
os-1-vm-fip --vars \"provider_network=...\""),
           crayons.yellow('ifr run --scenario pep8-tests \
--vars "project=/home/user/neutron"'),
           crayons.yellow('ifr list'),
           crayons.yellow('ifr show <scenario_name>'),
           crayons.yellow("ifr run --scenario pep8-tests --vars \
'project=/my/project execute=\"git \
checkout origin/some-branch; tox -e pep8\"'"))
    return message


def missing_arg(arg):
    """Missing arg message format."""
    message = """Please specify the argument {0} this way: infraform \
--vars \"{0}=...\"""".format(crayons.red(arg))
    return message


def missing_scenario(scenario):
    """Missing scenario message format."""
    message = """
Couldn't find the scenario: {0}

New scenarios should be added here: \
https://github.com/bregman-arie/infraform/tree/master/infraform/scenarios
""".format(crayons.red(scenario))
    return message


def missing_scenario_arg():
    """Missing scenario arg message format."""
    message = "Please specify which scenario to run with \
{}".format(crayons.red("--scenario <SCENARIO_NAME>"))
    return message



# Copyright 2019 Arie Bregman
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import logging
import os
from tabulate import tabulate

from infraform.utils import get_match_until_first_dot
from infraform.utils import get_description

LOG = logging.getLogger(__name__)

SCENARIOS_PATH = os.path.dirname(__file__) + '/../infraform/scenarios'


def list_scenarios():
    scenarios = []
    headers = ["Scenario Name", "Path", "Description"]
    for (dirpath, dirnames, filenames) in os.walk(SCENARIOS_PATH):
        for f in filenames:
            name = get_match_until_first_dot(f)
            scenario_path = dirpath + '/' + f
            with open(scenario_path, 'r') as f:
                description = get_description(f)
            scenarios.append([name, scenario_path, description])
    LOG.info(tabulate(scenarios, headers=headers))



# Copyright 2019 Arie Bregman
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import logging
import os
import subprocess
import sys

from infraform.exceptions import usage
from infraform.platforms.platform import Platform
from infraform.exceptions.utils import success_or_exit

LOG = logging.getLogger(__name__)


class Container(Platform):

    def __init__(self, args, binary, package):
        self.binary = binary
        self.package = package
        self.installation = "dnf install -y {0}\nsystemctl start {1}".format(
            self.package,
            self.binary)

        super(Container, self).__init__(args)

    # def clone_project(self):
    #    """Clones given project."""
    #    # Change project to path since docker run mounts volume
    #    # using the project argument
    #    clone_cmd = "git clone {}".format(self.vars['project'])
    #    subprocess.run(clone_cmd, shell=True, stdout=subprocess.DEVNULL)
    #    self.vars['project'] = os.getcwd() + '/' + self.vars['project_name']

    def verify_project_exists(self):
        if not os.path.isdir(self.vars['project']):
            success_or_exit(2, "Couldn't find project: {}".format(self.vars[
                'project']))

    def prepare(self):
        if self.image_not_exists() or (self.vars.get('override_image')):
            LOG.warning(
                "Couldn't find image: {}. Switching to image building".format(
                    self.vars['image']))
            dockerfile_path = self.write_dockerfile()
            self.build_image(dockerfile_path)
        if 'project' in self.vars:
            self.verify_project_exists()

    def run(self):
        """Run tests."""
        try:
            cmd = "{0} run -v {1}:/{2}:z {3} \
/bin/bash -c 'cd {2}; {4}'".format(
                self.binary,
                self.vars['project'],
                self.vars['project_name'],
                self.vars['image'],
                self.vars['execute'])
        except KeyError as e:
            LOG.error(usage.missing_arg(e.args[0]))
            sys.exit(2)
        print(cmd)
        res = subprocess.run(cmd, shell=True)
        success_or_exit(res.returncode)
        return res

    def image_not_exists(self):
        """Returns true if image exists."""
        res = subprocess.run(
            "{} inspect {}".format(self.binary, self.vars['image']),
            shell=True, stdout=subprocess.PIPE, stderr=subprocess.DEVNULL)
        return res.returncode

    def get_template(self, name):
        """Returns jinja2 Dockerfile template."""
        with open(name, 'r+') as open_f:
            template_content = open_f.read()
        return template_content

    def write_dockerfile(self, df_path="Dockerfile"):
        with open(df_path, 'w+') as f:
            f.write(self.vars['dockerfile'])
        return df_path

    def destroy(self):
        """Removes the container from the system."""
        res = subprocess.run("{} rm {}".format(self.binary,
                                               self.args['name']))
        if res.returncode != 0:
            sys.exit(2)
        return res

    def build_image(self, df_path):
        """Builds image given df path."""
        cmd = "{} build -f {} -t {} .".format(
            self.binary, df_path, self.vars['image'])
        res = subprocess.run(cmd, shell=True)
        if res.returncode != 0:
            sys.exit(2)
        return res



# Copyright 2019 Arie Bregman
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import setuptools

setuptools.setup(
    name='infraform',
    version='0.1',
    description='Creating the infra you need, not the one you want',
    author='Arie Bregman',
    author_email='abregman@redhat.com',
    packages=setuptools.find_packages(),
    python_requires='>3.0.0',
    include_package_data=True,
    entry_points={
        'console_scripts': ['infraform = infraform.cli.main:main',
                            'ifr = infraform.cli.main:main']}
)



# Copyright 2019 Arie Bregman
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import os

from infraform.platforms.platform import Platform
from infraform.exceptions.utils import success_or_exit

SCENARIOS_PATH = os.path.dirname(__file__) + '/../scenarios'


def guess_platform(scenario):
    """Try to figure out which platform the user should use or fail."""
    scenario_path, scenario_file = Platform.verify_scenario_exists(
        SCENARIOS_PATH, scenario)
    if scenario_file.endswith(".tf"):
        return "terraform"
    if scenario_file.endswith(".py"):
        return "python"
    if scenario_file.endswith(".sh"):
        return "shell"
    if os.path.dirname(scenario_path).split('/')[-1] == "podman":
        return "podman"
    if "docker-compose" in os.path.dirname(scenario_path).split('/'):
        return "docker_compose"
    success_or_exit(
        1,
        "Couldn't figure out which platform to use. Please specify --platform")



# Copyright 2019 Arie Bregman
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import logging
import importlib
import sys

from infraform.cli import utils
from infraform.exceptions.usage import missing_scenario_arg
from infraform.exceptions.utils import success_or_exit

LOG = logging.getLogger(__name__)


def main(args):
    """Runner main entry."""
    if not args.scenario and not args.vars:
        LOG.error(missing_scenario_arg())
        sys.exit(2)
    if args.scenario and not args.platform:
        args.platform = utils.guess_platform(args.scenario)
    if not args.scenario and not args.platform:
        success_or_exit(1, "Couldn't figure out which platform to use. \
Please specify --platform")
    Platform = getattr(importlib.import_module(
        "infraform.platforms.{}".format(args.platform)),
        args.platform.capitalize())
    platform = Platform(args=args)
    platform.prepare()
    platform.run()



# Copyright 2019 Arie Bregman
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import logging

from infraform.platforms.platform import Platform

LOG = logging.getLogger(__name__)


class Terraform(Platform):

    PACKAGE = 'terraform'
    BINARY = 'terraform'
    INSTALLATION = (
        "export version=0.12.10\nwget https://releases.hashicorp.com" +
        "/terraform/${version}/terraform_${version}_linux_amd64.zip" +
        "\nunzip terraform_${version}_linux_amd64.zip" +
        "\nsudo mv terraform /usr/local/bin/")

    def __init__(self, args):
        self.binary = self.BINARY
        self.package = self.PACKAGE
        self.installation = self.INSTALLATION
        super(Terraform, self).__init__(args)

    def prepare(self):
        self.execute_cmd('terraform init')

    def run(self):
        self.execute_cmd('terraform apply')



# Copyright 2019 Arie Bregman
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import logging
import os
import shutil
import subprocess

from infraform.platforms.platform import Platform
from infraform.exceptions.utils import success_or_exit

LOG = logging.getLogger(__name__)


class Docker_compose(Platform):

    PACKAGE = 'docker'
    BINARY = '/usr/local/bin/docker-compose'

    def __init__(self, args):
        self.binary = self.BINARY
        self.package = self.PACKAGE
        self.installation = "sudo dnf install docker"

        super(Docker_compose, self).__init__(args)

    def prepare(self):
        """Prepare environment for docker-compose execution."""
        new_dir = os.path.dirname(self.scenario_fpath).split('/')[-1]
        infraform_dir = os.path.expanduser('~') + '/.infraform/'
        self.execution_dir = infraform_dir + new_dir
        if os.path.isdir(self.execution_dir):
            shutil.rmtree(self.execution_dir)
        subprocess.call(['cp', '-r', os.path.dirname(self.scenario_fpath),
                         infraform_dir])

    def run(self):
        """Execution docker-compose."""
        cmd = self.vars['execute']
        res = subprocess.run(cmd, shell=True, cwd=self.execution_dir)
        success_or_exit(res.returncode)
        return res
